#include	"bc.h"
#include	"tnarray.h"

FlexObjectArrayBase::FlexObjectArrayBase( int _slot_size, bool _destruct )
{
	destruct = _destruct;
	num = 0;
	slot_size = _slot_size>0 ? _slot_size : 16;
	slot_num = 1;
	array = (void**)malloc( sizeof(void*) * slot_size);
	array[0] = 0;
	max_num = -1;
}

FlexObjectArrayBase::FlexObjectArrayBase( FlexObjectArrayBase &objs, int _slot_size, bool _destruct )
{
	destruct = _destruct;
	num = 0;
	slot_size = _slot_size;
	slot_num = 1;
	array = new void*[ slot_size ];
	array[0] = 0;

	for ( int i=0;i<objs.get_num();i++ ){
		add( objs.array[i] );
	}
}

FlexObjectArrayBase::~FlexObjectArrayBase()
{
	if ( array ){	// bug fix 1994.5.2 array==0̂Ƃɂclear()Ăł
		clear();
//		delete array;
		free( array );
	}
}

void FlexObjectArrayBase::clear( void )
{
	if ( destruct ){
		for (int i=0;i<num;i++){
			if ( array[i] ){
				delete_object( array[i] );
			}
		}
	}
	slot_num = 1;
	realloc();
	num = 0;
}

int FlexObjectArrayBase::realloc( void )
{
	void **_array = (void**)::realloc( array, sizeof( void* ) * slot_size * slot_num );
	if ( _array == 0 ){
		return -1;
	}
	array = _array;
	return 0;
}

int FlexObjectArrayBase::add( void *obj )
{
	if (max_num!=-1 && num>=max_num)
		return -1;
	if ( num >= slot_size * slot_num ){
		slot_num++;
		if ( realloc() == -1 ){
			return -1;
		}
	}
	array[num++] = obj;
	return 0;
}

void FlexObjectArrayBase::del( int i )
{
	if ( destruct ){
		delete_object( array[i] );
	}
#if 1
	memmove(&array[i], &array[i+1], (num-i-1)*sizeof(void*));
#else
	for (int j=i;j<num-1;j++){
		array[j] = array[j+1];
	}
#endif
	num--;
#if 0
	if ( slot_num > 1 && num < slot_size * ( slot_num - 1 ) ){
		slot_num--;
		realloc();
	}
#endif
}
void FlexObjectArrayBase::del(int bi, int ei)
{
	ei++;
	
	if ( destruct ){
		for (int i=bi;i<ei;i++){
			delete_object( array[i] );
		}
	}

#if 1
	memmove(&array[bi], &array[ei], (num-ei)*sizeof(void*));
#else
	int w = ei - bi;
	int n = num - ei;
	for (int j=0;j<n;j++){
		int i = bi + j;
		array[i] = array[i+w];
	}
#endif
	num -= ei-bi;
}
int FlexObjectArrayBase::insert( int i, void *obj )
{
	if ( i > num ){
		return -1;
	}
	if (max_num!=-1 && num>=max_num)
		return -1;

	if ( num >= slot_size * slot_num ){
		slot_num++;
		if ( realloc() == -1 ){
			return -1;
		}
	}
	memmove( &array[i+1], &array[i], ( num-i ) * sizeof( void *) );
	array[ i ] = obj;
	num++;
	return 0;
}
int FlexObjectArrayBase::replace( int i, void *obj )
{
	if ( i >= num ){
		return -1;
	}
	if ( destruct )
		delete_object( array[i] );
	array[i] = obj;
	return 0;
}
void FlexObjectArrayBase::exchange( int i, int j )
{
	void *obj = array[i];
	array[i] = array[j];
	array[j] = obj;
}

void FlexObjectArrayBase::move(int src, int dst)
{
	if (src<dst){
		if (src<0 || dst>=get_num())
			return;

		if (src+1==dst){
			exchange(src, src+1);
		} else {
			void *srcobj = array[src];
			memmove(&array[src], &array[src+1], (dst-src)*sizeof(void*));
			array[dst] = srcobj;
		}
	} else
	if (src>dst){
		if (src>=get_num() || dst<0)
			return;

		if (src-1==dst){
			exchange(src, dst);
		} else {
			void *srcobj = array[src];
			memmove(&array[dst+1], &array[dst], (src-dst)*sizeof(void*));
			array[dst] = srcobj;
		}
	} else {
		// no operation
	}
}

bool FlexObjectArrayBase::allocate(int size)
{
	if ( size <= slot_size * slot_num ){
		return true;	// Not changed (enough space);
	}
	slot_num = (size+slot_size-1) / slot_size;
	if (realloc()!=0){
		return false;	// error (not enough memory)
	}
	return true;	// reallocated
}

void FlexObjectArrayBase::discard( )
{
	int _destruct = destruct;
	destruct = 0;
	clear();
	destruct = _destruct;
}

void *FlexObjectArrayBase::discard( int i )
{
	int _destruct = destruct;
	destruct = 0;
	void *obj = array[i];
	del( i );
	destruct = _destruct;
	return obj;
}

void FlexObjectArrayBase::sort( int (*comp)( void **, void ** ) )
{
	qsort( array, num, sizeof( void * ), 
		(int(__cdecl *)(const void _FAR *, const void _FAR *))comp );
}

void *_bsearch( const void *key, const void *base, size_t nelem,
size_t width, int (* fcmp)(const void *, const void *))
{
  char  *kmin, *probe;
  int i, j;

  kmin = (char *) base;
  while (nelem > 0){
    i = nelem >> 1;
    probe = kmin + i * width;
    j = (*fcmp)(key, probe);
    if (j == 0)
      return(probe);
    else if (j < 0)
      nelem = i;
    else  {
      kmin = probe + width;
      nelem = nelem - i - 1;
    }
  }
  return kmin;
}

