#ifndef __tfilebuf_h
#define	__tfilebuf_h

class TTextFileBuffer {
protected:
	int fd;
	int curp;
	int Size;
public:
	TTextFileBuffer(int _fd, int size)
		:fd(_fd)
		,Size(size)
	{
		curp = Size;
	}
	virtual ~TTextFileBuffer(){}
	void flush()
	{
		curp = Size;
	}
	virtual int read() = 0;
	virtual void push_back(int c) = 0;
	virtual int remainBytes() = 0;
	virtual void *_get() = 0;
	void full()
	{
		curp = 0;
	}
protected:
	int read(void *buf, int len);
};

template <class T>
class TTextFileBufferT : public TTextFileBuffer {
typedef TTextFileBuffer super;
protected:
	T *Buffer;
public:
	TTextFileBufferT(int fd, int size)
		:super(fd, size)
	{
		Buffer = new T[size];
		Size = size;
	}
	virtual ~TTextFileBufferT()
	{
		if (Buffer)
			delete[] Buffer;
	}
	T &operator[](int index)
	{
		return Buffer[index];
	}
	T *get() const
		{ return Buffer; }
	virtual void *_get()
		{ return Buffer; }
	virtual void push_back(int c)
	{
		Buffer[ --curp ] = (T)c;
	}
	int remainBytes()
	{
		return (Size - curp)/sizeof(T);
	}
	virtual int read()
	{
		int d;
		if ( curp >= Size ){
			if ((d = (super::read(Buffer, Size*sizeof(T)))) <= 0)
				return -1;
			d = d/sizeof(T);
			if (d < Size){
				curp = Size - d;
				memmove(Buffer+curp, Buffer, d*sizeof(T));
			} else {
				curp = 0;
			}
			changeEndian();
		}
		return Buffer[curp++];
	}
protected:
	virtual void changeEndian(){}
	int read(void *buf, int len)
	{
		return super::read(buf, len);
	}
};

class TTextFileBufferUTF8 : public TTextFileBufferT<unsigned char> {
typedef TTextFileBufferT<unsigned char> super;
protected:
	wchar_t surrogate;
public:
	TTextFileBufferUTF8(int fd, int size)
		:super(fd, size)
	{
		surrogate = 0;
	}
	virtual int read();
};

class TTextFileBufferUTF16 : public TTextFileBufferT<wchar_t> {
typedef TTextFileBufferT<wchar_t> super;
protected:
	wchar_t surrogate;
public:
	TTextFileBufferUTF16(int fd, int size)
		:super(fd, size)
	{
		surrogate = 0;
	}
};

class TTextFileBufferUTF16BE : public TTextFileBufferUTF16 {
typedef TTextFileBufferUTF16 super;
public:
	TTextFileBufferUTF16BE(int fd, int size)
		:super(fd, size)
	{
	}
protected:
	virtual void changeEndian();
};

class TTextFileBufferAnsi : public TTextFileBufferT<unsigned char> {
typedef TTextFileBufferT<unsigned char> super;
protected:
public:
	TTextFileBufferAnsi(int fd, int size)
		:super(fd, size)
	{
	}
};

// Assumes the all code is Ascii text, not need the code conversion for ANSI/UTF-8
class TTextFileBufferAsc : public TTextFileBufferT<unsigned char> {
typedef TTextFileBufferT<unsigned char> super;
protected:
public:
	TTextFileBufferAsc(int fd, int size)
		:super(fd, size)
	{
	}
};

#endif	// __tfilebuf_h
