#include "tnwinx.h"
#pragma hdrstop
#include "utf.h"
#include "tnstr_temp.h"

/// __tnstrA ///
__tnstrA::__tnstrA(const wchar_t *s, bool utf8)
{
	if (utf8){
		setUTF8(s);
	} else {
		setAnsi(s);
	}
}
int __tnstrA::find(const char*pat)
{
	if (!buf)
		return -1;
	char *p = strstr(buf, pat);
	if (!p)
		return -1;
	return (int)(p-buf);
}

bool __tnstrA::LoadString(int id)
{
	//TODO: œKload@T
	static bool mutex = false;	// easy mutex for buf_LoadString
	for (;mutex;){ Sleep(1); }
	mutex = true;
	int size = ::LoadStringA(hTNInstance, id, buf_LoadString, sizeof(buf_LoadString)/sizeT);
	if (size==0){
		mutex = false;
		return false;
	}
	if (buf) delete[] buf;
	buf = new char[size+1];
	if (!buf){
		mutex = false;
		return false;
	}
	strcpy(buf, buf_LoadString);
	mutex = false;
	return true;
}

// Code Conversion //
// UTF16 >> UTF8
void __tnstrA::setUTF8(const wchar_t *utf16, int utf16_len)
{
	if (!utf16 || utf16_len==0){
		clear();
		return;
	}
	if (utf16_len<0){
		utf16_len = wcslen(utf16);
	}
	char *buf_new = new char[utf16_len*4+1];
	if (!buf_new)
		return;	// error
	UTF16toUTF8(utf16, utf16_len, buf_new);
	if (buf){
		delete[] buf;
	}
	buf = buf_new;
}

// UTF16 >> ANSI
void __tnstrA::setAnsi(const wchar_t *utf16, int utf16_len)
{
	if (!utf16 || utf16_len==0){
		clear();
		return;
	}
	if (utf16_len<0){
		utf16_len = wcslen(utf16);
	}
	int buf_size = utf16_len*2+1;
	char *buf_new = new char[buf_size];
	if (!buf_new)
		return;	// error
	int ret = WideCharToMultiByte(CP_ACP, 0, utf16, utf16_len, buf_new, buf_size, NULL, NULL );
	if (ret==0){
		delete[] buf_new;
		return;	// error
	}
	buf_new[ret] = '\0';
	if (buf){
		delete[] buf;
	}
	buf = buf_new;
}

__tnstrW __tnstrA::getWUTF8()
{
	__tnstrW ret;
	ret.setUTF8(*this);
	return ret;
}
__tnstrW __tnstrA::getWAnsi()
{
	__tnstrW ret;
	ret.setAnsi(*this);
	return ret;
}

/// __tnstrW ///
__tnstrW::__tnstrW(const char *s1, bool utf8)
{
	if (utf8){
		setUTF8(s1);
	} else {
		setAnsi(s1);
	}
}
int __tnstrW::find(const wchar_t*pat)
{
	if (!buf)
		return -1;
	wchar_t *p = wcsstr(buf, pat);
	if (!p)
		return -1;
	return (int)(p-buf);
}

bool __tnstrW::LoadString(int id)
{
	//TODO: œKload@T
	static bool mutex = false;	// easy mutex for buf_LoadString
	for (;mutex;){ Sleep(1); }
	mutex = true;
	int size = ::LoadStringW(hTNInstance, id, buf_LoadString, sizeof(buf_LoadString)/sizeT);
	if (size==0){
		mutex = false;
		return false;
	}
	if (buf) delete[] buf;
	buf = new wchar_t[size+1];
	if (!buf){
		mutex = false;
		return false;
	}
	wcscpy(buf, buf_LoadString);
	mutex = false;
	return true;
}

#ifndef __UTF8
bool __tnstrW::LoadText(int id)
{
	return GetTextResource(id, *this);
}
#endif

// Code Conversion //
// UTF-8 >> UTF-16
// utf8_len is byte unit.
void __tnstrW::setUTF8(const char *utf8, int utf8_len)
{
	if (!utf8 || utf8_len==0){
		clear();
		return;
	}
	if (utf8_len<0){
		utf8_len = strlen(utf8);
	}
	wchar_t *buf_new = new wchar_t[utf8_len*2+1];
	if (!buf_new)
		return;	// error
	UTF8toUTF16(utf8, utf8_len, buf_new);
	if (buf){
		delete[] buf;
	}
	buf = buf_new;
}

// ANSI >> UTF16
void __tnstrW::setAnsi(const char *ansi, int ansi_len)
{
	if (!ansi || ansi_len==0){
		clear();
		return;
	}
	if (ansi_len<0){
		ansi_len = strlen(ansi);
	}
	wchar_t *buf_new = new wchar_t[ansi_len*2+1];
	if (!buf_new)
		return;	// error
	UTF8toUTF16(ansi, ansi_len, buf_new);
	if (buf){
		delete[] buf;
	}
	buf = buf_new;
}

#include <StdCtrls.hpp>

/// _tnstrW ///
_tnstrW::_tnstrW(const WideString &ws)
	:super(ws.c_bstr())
{
}
_tnstrW::operator System::WideString () const
{
	return WideString(buf ? buf : buf_null);
}

_tnstrW &_tnstrW::operator = (const System::WideString &ws)
{
	set(ws.c_bstr());
	return *this;
}

__tnstrW tnsprintfW(const wchar_t *fmt, ...)
{
	va_list(ap);
	va_start(ap, fmt);
	wchar_t abuf[256];
	wchar_t *buf = NULL;
	int nsize = sizeof(abuf)/sizeof(wchar_t);
	int rsize = vsnwprintf(abuf, nsize-1, fmt, ap);
	if (rsize>=nsize-1){
		nsize = vsnwprintf(NULL, 0, fmt, ap)+1;
		buf = new wchar_t[nsize];
		if (buf){
			vsnwprintf(buf, nsize, fmt, ap);
		} else {
			abuf[0] = '\0';
		}
	}
	va_end(ap);
	if (buf){
		auto_ptr<wchar_t> _buf(buf);
		return __tnstrW(buf);
	} else {
		return __tnstrW(abuf);
	}
}


