#include "tndefs.h"
#pragma hdrstop

#include "utf.h"
typedef unsigned char byte;

int UTF16toUTF8( const wchar_t *src, int srclen, char *dst )
{
	const wchar_t *end;
	if ( srclen == -1 ){
		end = (wchar_t*)-1;
	} else {
		end = src + srclen;
	}
	int size = 0;
	while (src<end){
		long c = *src++;
		if (c == 0)
			break;	// end of string
		if ( c >= 0xD800 && c <= 0xDBFF ){
			wchar_t wc = *src++;
			if ( wc >= 0xDC00 && wc <= 0xDFFF ){
				c = (((c-0xD800)<<10) | (wc - 0xDC00)) + 0x10000;
			} else {
				// illegal code
				continue;	// skip
			}
		}

		// change UTF-32 to UTF-8
		int w;
		if ( c < 0x80 ){
			// 1 octet
			w = 1;
			*dst++ = (byte)c;
		} else {
			if ( c < 0x800 ){
				w = 2;
				*dst++ = (byte)((c>>6)+0xC0);
			} else {
				if ( c < 0x10000 ){
					w = 3;
					*dst++ = (byte)((c>>12)+0xE0);
				} else {
					w = 4;
					*dst++ = (byte)((c>>18)+0xF0);
					*dst++ = (byte)(((c>>12)&0x3F)+0x80);
				}
				*dst++ = (byte)(((c>>6)&0x3F)+0x80);
			}
			*dst++ = (byte)((c&0x3F)+0x80);
		}
		size += w;
	}
	*dst = '\0';
	return size;
}
int UTF8toUTF16( const char *src, int srclen, wchar_t *dst )
{
	const char *end;
	if ( srclen == -1 ){
		end = (char*)-1;
	} else {
		end = src + srclen;
	}
	int size = 0;
	while (src<end){
		long c = *(byte*)src++;
		if (c == 0)
			break;	// end of string
		if (c & 0x80){
			unsigned long diff;
			diff = (0xC0<<6);
			c = (c<<6) | (*(byte*)src++ & 0x3F);
			if ( c >= (0xE0<<6) ){
				diff = (0xE0<<12);
				c = (c<<6) | (*(byte*)src++ & 0x3F);
				if ( c >= (0xF0<<12) ){
					diff = (0xF0<<18);
					c = (c<<6) | (*(byte*)src++ & 0x3F);
				}
			}
			c -= diff;
		}
		// change UTF-32 to UTF-16 //
		if (c <= 0xFFFF){
			*dst++ = (wchar_t)c;
			size++;
		} else {
			// surrogate
			*dst++ = (wchar_t)(((c - 0x10000)>>10) + 0xD800);
			*dst++ = (wchar_t)((c & 0x3FF) + 0xDC00);
			size+=2;
		}
	}
	*dst = '\0';
	return size;
}
unsigned long UTF8toUTF16( const char *src, const char **next )
{
	unsigned long c = *(byte*)src;
	if (c == 0){
		*next = src;
		return 0;	// end of string
	}
	src++;
	if (c & 0x80){
		unsigned long diff;
		diff = (0xC0<<6);
		byte cc = *(byte*)src;
		if (!cc){
			*next = src;
			return 0;	// illegal sequence
		}
		src++;
		c = (c<<6) | (cc & 0x3F);
		if ( c >= (0xE0<<6) ){
			diff = (0xE0<<12);
			cc = *(byte*)src;
			if (!cc){
				*next = src;
				return 0;	// illegal sequence
			}
			src++;
			c = (c<<6) | (cc & 0x3F);
			if ( c >= (0xF0<<12) ){
				diff = (0xF0<<18);
				cc = *(byte*)src;
				if (!cc){
					*next = src;
					return 0;	// illegal sequence
				}
				src++;
				c = (c<<6) | (cc & 0x3F);
			}
		}
		c -= diff;
	}
	// change UTF-32 to UTF-16 //
	if (c >= 0x10000){
		// surrogate (not yet debugged)
		c = ((((c - 0x10000)>>10) + 0xD800)<<16) | (wchar_t)((c & 0x3FF) + 0xDC00);
	}
	*next = src;
	return c;
}

const char *utf8CharNext(const char *s)
{
	unsigned long c = *(byte*)s;
	if (*s == 0){
		return s;	// end of string
	}
	s++;
	if (c & 0x80){
		unsigned long diff;
		diff = (0xC0<<6);
		byte cc = *(byte*)s;
		if (!cc){
			return s;	// illegal sequence
		}
		s++;
		c = (c<<6) | (cc & 0x3F);
		if ( c >= (0xE0<<6) ){
			diff = (0xE0<<12);
			cc = *(byte*)s;
			if (!cc){
				return s;	// illegal sequence
			}
			s++;
			c = (c<<6) | (cc & 0x3F);
			if ( c >= (0xF0<<12) ){
				diff = (0xF0<<18);
				cc = *(byte*)s;
				if (!cc){
					return s;	// illegal sequence
				}
				s++;
				c = (c<<6) | (cc & 0x3F);
			}
		}
		c -= diff;
	}
	// change UTF-32 to UTF-16 //
	if (c >= 0x10000){
		// surrogate (not yet debugged)
		c = ((((c - 0x10000)>>10) + 0xD800)<<16) | (wchar_t)((c & 0x3FF) + 0xDC00);
	}
	return s;
}

